package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.MethodChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * 
 * Checkers that verifies that a method does not throw a null pointer exception.
 * 
 * @author Romain Delamare
 * 
 */
public class NPEMethodChecker extends MethodChecker {

	private static final Logger	LOGGER	= LoggerFactory
												.getLogger(NPEMethodChecker.class);

	public NPEMethodChecker(final IInstanceProvider provider,
			final Method method) {
		super(provider, method);
	}

	public void check() {
		if (method != null) {
			final Object instance = provider.getInstance(method
					.getDeclaringClass());
			try {
				method.invoke(instance, Utils
						.generateDefaultValuesParameters(method
								.getParameterTypes()));
			} catch (final IllegalAccessException e) {
				LOGGER.error(
						"Error during invocation of {}: illegal arguments.",
						method);
			} catch (final IllegalArgumentException e) {
				LOGGER.error("Error during invocation of {}: illegal access.",
						method);
			} catch (final InvocationTargetException e) {
				if (e.getCause() instanceof NullPointerException)
					throw new EasycoverageAssertionError(
							"Null pointer exception on " + method.getName()
									+ " when invoked with default values.",
							e.getCause());
			}
		}
	}

	@Override
	protected boolean shouldCheckMethod() {
		return true;
	}

}
